import React from 'react';
import { useState, lazy, Suspense, useEffect } from 'react';
import { BrowserRouter as Router, Routes, Route, useParams, Link } from 'react-router-dom';
import { HelmetProvider } from 'react-helmet-async';
import { FileAudio, Languages, Mic, Moon, Sun, Sparkles } from 'lucide-react';
import { AudioRecorder } from './components/AudioRecorder';
import { FileUpload } from './components/FileUpload';
import { Features as HomeFeatures } from './components/Features';
import { Footer } from './components/Footer';
import { TranscriptionOutput } from './components/TranscriptionOutput';
import { SkyBackground } from './components/SkyBackground';
import { SEOHead } from './components/SEOHead';
import { GeoLocationHandler } from './components/GeoLocationHandler';
import { LocalizedContent } from './components/LocalizedContent';
import { LocalBusiness } from './components/LocalBusiness';
import { FAQSchema, comprehensiveFAQs } from './components/FAQSchema';
import { Breadcrumbs } from './components/Breadcrumbs';
import { ServiceSchema, HowToSchema } from './components/StructuredData';
import { useSEO, generateSEOConfig } from './hooks/useSEO';
import { transcribeAudio, generateSpeech } from './lib/openai';
import type { TranscriptionResult, ExportFormat } from './types'
import { trackEvent } from './lib/analytics';

// Lazy load pages
const Blog = lazy(() => import('./pages/Blog').then(module => ({ default: module.Blog })));
const BlogPost = lazy(() => import('./components/BlogPost').then(module => ({ default: module.BlogPost })));
const Features = lazy(() => import('./pages/Features').then(module => ({ default: module.Features })));
const Pricing = lazy(() => import('./pages/Pricing').then(module => ({ default: module.Pricing })));
const FAQ = lazy(() => import('./pages/FAQ').then(module => ({ default: module.FAQ })));

// Import blog posts data
const blogPosts = [
  {
    id: '1',
    slug: 'how-to-convert-audio-to-text-complete-guide-2025',
    title: 'How to Convert Audio to Text: Complete Guide 2025',
    excerpt: 'Learn the best methods to convert audio files to text in 2025. Compare free vs paid tools, accuracy rates, and discover which solution works best for your needs.',
    content: 'Complete content would be here...',
    author: 'AudioToText Team',
    publishedAt: '2025-01-15',
    readTime: 8,
    tags: ['Audio to Text', 'Transcription', 'AI Technology', 'Tutorial'],
    metaDescription: 'Complete guide to converting audio to text in 2025. Learn about the best tools, accuracy factors, and step-by-step instructions for perfect transcription.',
    keywords: ['audio to text', 'speech to text', 'transcription software', 'convert audio to text', 'AI transcription'],
    featuredImage: 'https://images.unsplash.com/photo-1589903308904-1010c2294adc?auto=format&fit=crop&w=1200&h=600&q=80'
  }
  // Additional blog posts would be imported here
];

interface GeoLocation {
  country: string;
  countryCode: string;
  region: string;
  city: string;
  timezone: string;
  currency: string;
  language: string;
}

function HomePage() {
  const [isProcessing, setIsProcessing] = useState(false);
  const [processingStatus, setProcessingStatus] = useState<string>('');
  const [result, setResult] = useState<TranscriptionResult | null>(null);
  const [activeTab, setActiveTab] = useState<'upload' | 'record'>('upload');
  const [selectedLanguage, setSelectedLanguage] = useState<string>('en');
  const [darkMode, setDarkMode] = useState<boolean>(false);
  const [geoLocation, setGeoLocation] = useState<GeoLocation | null>(null);

  // SEO hook
  useSEO(generateSEOConfig('home'));

  const handleFileSelect = async (file: File) => {
    try {
      setIsProcessing(true);
      setProcessingStatus('Uploading audio file...');
      
      await new Promise(resolve => setTimeout(resolve, 1000));
      setProcessingStatus('Transcribing audio...');
      
      const transcription = await transcribeAudio(file, {
        language: selectedLanguage,
      });

      setProcessingStatus('Finalizing results...');
      await new Promise(resolve => setTimeout(resolve, 500));
      
      setResult(transcription);
      
      // Track successful transcription
      trackEvent('transcription', 'complete', selectedLanguage, file.size);
    } catch (error) {
      console.error('Transcription error:', error);
      // Track transcription error
      trackEvent('transcription', 'error', error instanceof Error ? error.message : 'Unknown error');
    } finally {
      setIsProcessing(false);
      setProcessingStatus('');
    }
  };

  const handleRecordingComplete = async (blob: Blob) => {
    const file = new File([blob], 'recording.webm', { type: 'audio/webm' });
    trackEvent('recording', 'complete', 'webm', blob.size);
    await handleFileSelect(file);
  };

  const handleExport = (format: ExportFormat) => {
    if (!result) return;

    let content = '';
    let mimeType = '';
    let filename = '';

    switch (format) {
      case 'txt':
        content = result.text;
        mimeType = 'text/plain';
        filename = 'transcription.txt';
        break;
      case 'srt':
        content = result.segments?.map((segment, i) => {
          const start = new Date(segment.start * 1000).toISOString().substr(11, 12).replace('.', ',');
          const end = new Date(segment.end * 1000).toISOString().substr(11, 12).replace('.', ',');
          return `${i + 1}\n${start} --> ${end}\n${segment.text}\n`;
        }).join('\n') || '';
        mimeType = 'text/plain';
        filename = 'transcription.srt';
        break;
      default:
        return;
    }

    const blob = new Blob([content], { type: mimeType });
    const url = URL.createObjectURL(blob);
    const a = document.createElement('a');
    a.href = url;
    a.download = filename;
    a.click();
    URL.revokeObjectURL(url);
  };

  const handleTextToSpeech = async (text: string) => {
    try {
      const audioUrl = await generateSpeech(text);
      const audio = new Audio(audioUrl);
      audio.play();
    } catch (error) {
      console.error('Text-to-speech error:', error);
    }
  };

  const toggleDarkMode = () => {
    setDarkMode(!darkMode);
    document.documentElement.classList.toggle('dark');
  };

  const handleLocationDetected = (location: GeoLocation) => {
    setGeoLocation(location);
  };

  // Service schema for home page
  const serviceSchema = {
    "@type": "Service",
    "name": "AI-Powered Audio Transcription Service",
    "description": "Professional audio-to-text conversion with 95%+ accuracy, supporting 50+ languages and real-time processing.",
    "provider": {
      "@type": "Organization",
      "name": "AudioToText.xyz"
    },
    "serviceType": "Audio Transcription",
    "areaServed": "Worldwide",
    "offers": {
      "@type": "Offer",
      "url": "https://audiototext.xyz",
      "priceCurrency": "USD",
      "price": "0",
      "priceValidUntil": "2025-12-31",
      "itemCondition": "https://schema.org/NewCondition",
      "availability": "https://schema.org/InStock",
      "description": "Free tier with premium options available"
    }
  };

  // How-to schema for the transcription process
  const howToSchema = {
    "name": "How to Convert Audio to Text with AudioToText.xyz",
    "description": "Step-by-step guide to transcribe audio files using our AI-powered service",
    "totalTime": "PT5M",
    "estimatedCost": "0",
    "supply": ["Audio file", "Internet connection"],
    "tool": ["Web browser", "AudioToText.xyz platform"],
    "steps": [
      {
        "name": "Upload Audio File",
        "text": "Click on the upload area and select your audio file from your device. Supported formats include MP3, WAV, AAC, FLAC, and M4A."
      },
      {
        "name": "Select Language",
        "text": "Choose the language of your audio from our list of 50+ supported languages for optimal accuracy."
      },
      {
        "name": "Start Transcription",
        "text": "Click the transcribe button and wait while our AI processes your audio file. This typically takes 2-3 minutes."
      },
      {
        "name": "Review and Edit",
        "text": "Review the generated transcript and make any necessary edits using our built-in editor."
      },
      {
        "name": "Export Results",
        "text": "Download your transcription in your preferred format: TXT, SRT, DOCX, or VTT."
      }
    ]
  };

  return (
    <GeoLocationHandler onLocationDetected={handleLocationDetected}>
      <LocalizedContent 
        countryCode={geoLocation?.countryCode || 'US'}
        language={geoLocation?.language || 'en'}
        currency={geoLocation?.currency || 'USD'}
      >
        <div className={`min-h-screen relative ${darkMode ? 'dark' : ''}`}>
          {/* SEO Components */}
          <SEOHead
            schema={serviceSchema}
            geo={geoLocation ? {
              country: geoLocation.countryCode,
              region: geoLocation.region,
              placename: geoLocation.city
            } : undefined}
          />
          
          {geoLocation && (
            <LocalBusiness
              city={geoLocation.city}
              region={geoLocation.region}
              country={geoLocation.country}
              countryCode={geoLocation.countryCode}
            />
          )}
          
          <ServiceSchema
            name="AI-Powered Audio Transcription"
            description="Convert speech to text with 95%+ accuracy using advanced AI technology"
            provider="AudioToText.xyz"
            areaServed="Worldwide"
          />
          
          <HowToSchema {...howToSchema} />
          
          <FAQSchema faqs={comprehensiveFAQs} />

          <SkyBackground darkMode={darkMode} />
          
          {/* Theme toggle button */}
          <button 
            onClick={toggleDarkMode}
            className="fab" 
            style={{ bottom: '30px', right: '30px' }}
            aria-label="Toggle theme"
          >
            {darkMode ? <Sun className="w-6 h-6" /> : <Moon className="w-6 h-6" />}
          </button>
          
          <div className="relative max-w-4xl mx-auto py-6 sm:py-12 px-3 sm:px-4 animate-fade-in">
            <div className="text-center mb-8">
              <div className="flex items-center justify-center mb-3">
                <Sparkles className="w-10 h-10 text-secondary mr-2" />
                <h1 className="text-4xl sm:text-6xl font-extrabold gradient-text mb-0 tracking-tight [text-shadow:_0_1px_2px_rgb(0_0_0_/_5%)]">
                  AudioToText.xyz
                </h1>
              </div>
              <p className="text-lg sm:text-xl text-primary-700 font-medium">
                Professional AI-Powered Audio Transcription
              </p>
              <div className="flex justify-center mt-4">
                <a 
                  href="https://www.producthunt.com/posts/audiototext-xyz?embed=true&utm_source=badge&utm_medium=badge&utm_souce=badge-audiototext&#0045;xyz" 
                  target="_blank" 
                  rel="noopener noreferrer"
                  className="hover-lift"
                >
                  <img 
                    src="https://api.producthunt.com/widgets/embed-image/v1/featured.svg?post_id=880800&theme=light&t=1739555083234" 
                    alt="AudioToText&#0046;xyz - Professional&#0032;AI&#0045;Powered&#0032;Audio&#0032;Transcription | Product Hunt" 
                    className="h-[54px] w-[250px]"
                    width="250" 
                    height="54" 
                  />
                </a>
              </div>
            </div>

            <div className="glass neumorphic rounded-xl sm:rounded-2xl p-4 sm:p-8 mb-6 sm:mb-8 transition-all duration-300">
              <div className="flex flex-col sm:flex-row gap-2 sm:gap-4 mb-4 sm:mb-6 p-1 bg-white/30 rounded-lg">
                <button
                  onClick={() => setActiveTab('upload')}
                  className={`tab-button ${activeTab === 'upload' ? 'active' : ''}`}
                >
                  <FileAudio className="w-5 h-5" />
                  Upload File
                </button>
                <button
                  onClick={() => setActiveTab('record')}
                  className={`tab-button ${activeTab === 'record' ? 'active' : ''}`}
                >
                  <Mic className="w-5 h-5" />
                  Record Audio
                </button>
              </div>

              <div className="mb-6">
                <div className="flex items-center gap-2 mb-4">
                  <Languages className="w-5 h-5 text-secondary" />
                  <select
                    value={selectedLanguage}
                    onChange={(e) => setSelectedLanguage(e.target.value)}
                    className="input-control focus-ring"
                  >
                    <option value="en">English</option>
                    <option value="hi">Hindi</option>
                    <option value="ja">Japanese</option>
                    <option value="es">Spanish</option>
                    <option value="fr">French</option>
                    <option value="de">German</option>
                    <option value="it">Italian</option>
                    <option value="nl">Dutch</option>
                    <option value="pt">Portuguese</option>
                    <option value="ru">Russian</option>
                  </select>
                </div>

                {activeTab === 'upload' ? (
                  <FileUpload
                    onFileSelect={handleFileSelect}
                    isProcessing={isProcessing}
                    processingStatus={processingStatus}
                  />
                ) : (
                  <AudioRecorder
                    onRecordingComplete={handleRecordingComplete}
                    isProcessing={isProcessing}
                    processingStatus={processingStatus}
                  />
                )}
                {isProcessing && (
                  <div className="mt-4">
                    <div className="h-2 w-full bg-gray-200 rounded-full overflow-hidden">
                      <div 
                        className="h-full bg-gradient-to-r from-secondary to-accent rounded-full transition-all duration-500"
                        style={{ 
                          width: processingStatus.includes('Uploading') ? '33%' : 
                                 processingStatus.includes('Transcribing') ? '66%' : 
                                 processingStatus.includes('Finalizing') ? '90%' : '0%' 
                        }}
                      />
                    </div>
                    <p className="text-sm text-primary-700 mt-2 text-center font-medium">{processingStatus}</p>
                  </div>
                )}
              </div>
            </div>

            {result && (
              <div className="animate-fade-in">
                <TranscriptionOutput
                  result={result}
                  onExport={handleExport}
                  onTextToSpeech={handleTextToSpeech}
                />
              </div>
            )}
            
            <HomeFeatures />
          </div>
          <Footer />
        </div>
      </LocalizedContent>
    </GeoLocationHandler>
  );
}

function BlogPostPage() {
  const { slug } = useParams();
  const post = blogPosts.find(p => p.slug === slug);
  
  if (!post) {
    return (
      <div className="min-h-screen flex items-center justify-center">
        <div className="text-center">
          <h1 className="text-2xl font-bold text-gray-900 mb-4">Post Not Found</h1>
          <Link to="/blog" className="text-secondary hover:underline">
            Back to Blog
          </Link>
        </div>
      </div>
    );
  }
  
  return (
    <>
      <Breadcrumbs />
      <BlogPost post={post} />
    </>
  );
}

function App() {
  return (
    <HelmetProvider>
      <Router>
        <Suspense fallback={
          <div className="min-h-screen flex items-center justify-center bg-gray-50">
            <div className="animate-spin rounded-full h-12 w-12 border-4 border-secondary border-t-transparent"></div>
          </div>
        }>
          <Routes>
            <Route path="/" element={<HomePage />} />
            <Route path="/blog" element={<Blog />} />
            <Route path="/blog/:slug" element={<BlogPostPage />} />
            <Route path="/features" element={<Features />} />
            <Route path="/pricing" element={<Pricing />} />
            <Route path="/faq" element={<FAQ />} />
            
            {/* Local/Geo-specific routes */}
            <Route path="/:countryCode" element={<HomePage />} />
            <Route path="/:countryCode/:city" element={<HomePage />} />
            <Route path="/:countryCode/:city/:service" element={<HomePage />} />
          </Routes>
        </Suspense>
      </Router>
    </HelmetProvider>
  );
}

export default App;